function [ trainIdx ] = define_trainingset( Labels, ratio )
%DEFINE_TRAININGSET Summary of this function goes here
%   Labels - Nx1 vector, of true labels
%   ratio  - the fraction of labels to be used as training set
%
%   trainIdx - the indices of the labels to be used as training set

if ratio == 1
    trainIdx = true(size(Labels));
    return;
end


% Get generator settings (for reproducibility)
warning('off','MATLAB:RandStream:ActivatingLegacyGenerators')
s = rng;
rng(0,'v4')

trainIdx = false(size(Labels));
uniqueLabels = unique(Labels);

for i = 1:length(uniqueLabels)
    % Count the current label's instances
    iscurlabel = (Labels == uniqueLabels(i)); 
    nLabeli = nnz(iscurlabel);                
    % Select the ratio of them randomly
    sel = randsample(find(iscurlabel),ceil(ratio*nLabeli));    
    % Put them in the training set
    trainIdx(sel) = 1;                                    
end


% Restore previous generator settings.
rng(s);
warning('on','MATLAB:RandStream:ActivatingLegacyGenerators')



end

