% 
%   MLCV Summer 2016
%   Homework Assignment 05 - Gaussian Processes
%
%   Exercise 5) Classification of bank-authentication dataset
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Read in the dataset
datasetName = 'banknote_auth';
datasetsDir = fullfile(pwd, '../datasets');

datasetDir = fullfile(datasetsDir, datasetName);
inputsFile = fullfile(datasetDir, [datasetName '_data.csv']);
labelsFile = fullfile(datasetDir, [datasetName '_labels.csv']);

X = importdata(inputsFile);
Y = importdata(labelsFile);

% Make labels -1 and +1
labels = unique(Y);
Ytemp = Y;
Y(Ytemp==labels(1)) = -1;
Y(Ytemp==labels(2)) = 1;
clear Ytemp;

%% Split in training and testing sets
trainingRatio = 0.5;
[ Xtrain, Ytrain, Xtest, Ytest ] = define_learningset(X, Y, trainingRatio);

% Display the training and testing sets
figure;
subplot(2,2,1);
hold on; axis square;
idx = Ytrain==1;
plot(Xtrain(idx,1), Xtrain(idx,2),'b*');
plot(Xtrain(~idx,1), Xtrain(~idx,2),'r*');
title('Training set');

subplot(2,2,2);
hold on; axis square;
idx = Ytest==1;
plot(Xtest(idx,1), Xtest(idx,2),'b*');
plot(Xtest(~idx,1), Xtest(~idx,2),'r*');
title('Testing set');


%% Compute class means and covariances in the training set 
idxPos = (Ytrain == 1);  
idxNeg = ~idxPos;

% number of data points from each class
n1 = sum(idxPos);  
n2 = sum(idxNeg);

% the two covariance matrices
S1 = cov(Xtrain(idxPos,:));
S2 = cov(Xtrain(idxNeg,:));

% the two means          
m1 = mean(Xtrain(idxPos,:));
m2 = mean(Xtrain(idxNeg,:));

%% Define a mean function, a covariance function, a likelihood function
% initialize the hyperparams, choose an inference method (Laplace, EP)

meanfunc = @meanConst;
covfunc = @covSEard; % squared-exp. kernel with Automatic Relevance Determination
likfunc = @likErf; % @likErf , @likLogistic , @likUni
ell = 1.0; % length scale
sf = 1.0;  % magnitude
hyp.mean = 0;
hyp.cov = log([ell ell ell ell sf]); % length scale for every input dimension and sf
inffunc = @infLaplace; % @infLaplace , @infEP , @infFITC_Laplace , infFITC_EP

%% Find optimal hyperparams
hyp = minimize(hyp, @gp, -40, inffunc, meanfunc, covfunc, likfunc, Xtrain, Ytrain);

%% Predict for the test inputs
% Find output and latent means and covariances and probabilities
[ymu, ys2, fmu, fs2, lp] = gp(hyp, inffunc, meanfunc, covfunc, likfunc, Xtrain, Ytrain, Xtest, Ytest);
probs = exp(lp);
err = mean(probs < 0.5);

%% Compute probabilities for 2d contour plot

xx = linspace(min(Xtest(:,1)), max(Xtest(:,1)), 40);
yy = linspace(min(Xtest(:,2)), max(Xtest(:,2)), 40);
[t1, t2] = meshgrid(xx,yy);
t = [t1(:) t2(:)]; n = length(t);
tmm = bsxfun(@minus, t, m1(1:2));
p1 = n1*exp(-sum(tmm*inv(S1(1:2, 1:2)).*tmm/2,2))/sqrt(det(S1(1:2, 1:2)));
tmm = bsxfun(@minus, t, m2(1:2));
p2 = n2*exp(-sum(tmm*inv(S2(1:2, 1:2)).*tmm/2,2))/sqrt(det(S2(1:2, 1:2)));


%% Plot contours
figure; 
colormap(jet)
%set(gca, 'FontSize', 24)
contourLevels = 0.0:0.1:1.0;
contourf(t1, t2, reshape(p2./(p1+p2), size(t1)), contourLevels,'ShowText','on') 
hold on
[c, h] = contour(t1, t2, reshape(p2./(p1+p2), size(t1)), [0.5 0.5]);
set(h, 'LineWidth', 2)
colorbar
grid

% Plot testing points on top
hold on
idx = Ytest==1;
plot(Xtest(idx,1), Xtest(idx,2),'b*'); hold on
plot(Xtest(~idx,1), Xtest(~idx,2),'r*');
title('Testing set');


